@@ -26,6 +26,7 @@ except ModuleNotFoundError:
                    "To enable it, install u8darts[fbprophet] or u8darts[all].")
 
 try:
+    from .block_rnn_model import BlockRNNModel
     from .rnn_model import RNNModel
     from .tcn_model import TCNModel
     from .nbeats import NBEATSModel
@@ -35,9 +36,16 @@ except ModuleNotFoundError:
     logger.warning("Support Torch based models not available. To enable it, install u8darts[torch] or u8darts[all].")
 
 # Regression
-from .standard_regression_model import StandardRegressionModel
+from .linear_regression_model import LinearRegressionModel
+from .random_forest import RandomForest
+from .regression_model import RegressionModel
 
 # Ensembling
 from .ensemble_model import EnsembleModel
 from .baselines import NaiveEnsembleModel
 from .regression_ensemble_model import RegressionEnsembleModel
+
+# Filtering
+from .filtering_model import MovingAverage
+from .kalman_filter import KalmanFilter
+from .gaussian_process_filter import GaussianProcessFilter
