@@ -2,6 +2,8 @@ import unittest
 import logging
 
 from darts.dataprocessing.transformers import BaseDataTransformer
+from darts.utils.timeseries_generation import constant_timeseries
+from darts import TimeSeries
 
 
 class BaseDataTransformerTestCase(unittest.TestCase):
@@ -11,22 +13,22 @@ class BaseDataTransformerTestCase(unittest.TestCase):
     def setUpClass(cls):
         logging.disable(logging.CRITICAL)
 
-    class DataTransformerMock(BaseDataTransformer[str]):
+    class DataTransformerMock(BaseDataTransformer):
         def __init__(self):
             super().__init__(name="DataTransformerMock")
             self.transform_called = False
 
-        def transform(self, data: str, *args, **kwargs) -> str:
-            self.transform_called = True
-            return data + "transformed"
+        @staticmethod
+        def ts_transform(series: TimeSeries) -> TimeSeries:
+            return series + 10
 
     def test_input_transformed(self):
         # given
-        test_input = "test"
+        test_input = constant_timeseries(value=1)
         mock = self.DataTransformerMock()
 
         # when
         transformed = mock.transform(test_input)
 
-        expected = "testtransformed"
+        expected = constant_timeseries(value=11)
         self.assertEqual(transformed, expected)
