@@ -6,7 +6,7 @@ import pandas as pd
 
 from ..timeseries import TimeSeries
 from ..logging import raise_log, get_logger, raise_if_not, raise_if
-from typing import List, Callable, TypeVar, Union
+from typing import List, Callable, TypeVar, Iterator, Tuple
 from IPython import get_ipython
 from tqdm import tqdm
 from tqdm.notebook import tqdm as tqdm_notebook
@@ -14,6 +14,7 @@ from functools import wraps
 from types import SimpleNamespace
 from inspect import signature, Parameter, getcallargs
 from enum import Enum
+from joblib import Parallel, delayed
 
 logger = get_logger(__name__)
 
@@ -40,17 +41,14 @@ def retain_period_common_to_all(series: List[TimeSeries]) -> List[TimeSeries]:
     """
     Trims all series in the provided list, if necessary, so that the returned time series have
     a common span (corresponding to largest time sub-interval common to all series).
-
     Parameters
     ----------
     series
         The list of series to consider.
-
     Raises
     ------
     ValueError
         If no common time sub-interval exists
-
     Returns
     -------
     List[TimeSeries]
@@ -66,18 +64,18 @@ def retain_period_common_to_all(series: List[TimeSeries]) -> List[TimeSeries]:
     return list(map(lambda s: s.slice(last_first, first_last), series))
 
 
-def _build_tqdm_iterator(iterable, verbose):
+def _build_tqdm_iterator(iterable, verbose, **kwargs):
     """
     Build an iterable, possibly using tqdm (either in notebook or regular mode)
-
     Parameters
     ----------
     iterable
     verbose
+    total
+        Length of the iterator, helps in cases where tqdm is not detecting the total length.
 
     Returns
     -------
-
     """
 
     def _isnotebook():
@@ -94,9 +92,10 @@ def _build_tqdm_iterator(iterable, verbose):
 
     if verbose:
         if _isnotebook():
-            iterator = tqdm_notebook(iterable)
+            iterator = tqdm_notebook(iterable, **kwargs)
         else:
-            iterator = tqdm(iterable)
+            iterator = tqdm(iterable, **kwargs)
+
     else:
         iterator = iterable
     return iterator
@@ -113,22 +112,18 @@ def _with_sanity_checks(*sanity_check_methods: str) -> Callable[[Callable[[A, B]
     Decorator allowing to specify some sanity check method(s) to be used on a class method.
     The decorator guarantees that args and kwargs from the method to sanitize will be available in the
     sanity check methods as specified in the sanitized method's signature, irrespective of how it was called.
-
     Parameters
     ----------
     *sanity_check_methods
         one or more sanity check methods that will be called with all the parameter of the decorated method.
-
     Returns
     -------
     A Callable corresponding to the decorated method.
-
     Examples
     --------
     class Model:
         def _a_sanity_check(self, *args, **kwargs):
             raise_if_not(kwargs['b'] == kwargs['c'], 'b must equal c', logger)
-
         @_with_sanity_checks("_a_sanity_check")
         def fit(self, a, b=0, c=0):
             # at this point we can safely assume that 'b' and 'c' are equal...
@@ -162,7 +157,6 @@ def _with_sanity_checks(*sanity_check_methods: str) -> Callable[[Callable[[A, B]
 def _historical_forecasts_general_checks(series, kwargs):
     """
     Performs checks common to ForecastingModel and RegressionModel backtest() methods
-
     Parameters
     ----------
     series
@@ -188,7 +182,7 @@ def _historical_forecasts_general_checks(series, kwargs):
     # check start parameter
     if hasattr(n, 'start'):
         if isinstance(n.start, float):
-            raise_if_not(n.start >= 0.0 and n.start < 1.0, '`start` should be between 0.0 and 1.0.', logger)
+            raise_if_not(0.0 <= n.start < 1.0, '`start` should be between 0.0 and 1.0.', logger)
         elif isinstance(n.start, pd.Timestamp):
             raise_if(n.start not in series, '`start` timestamp must be an entry in the time series\' time index')
             raise_if(n.start == series.end_time(), '`start` timestamp is the last timestamp of the series', logger)
@@ -210,6 +204,34 @@ def _historical_forecasts_general_checks(series, kwargs):
     overlap_end = n.overlap_end
 
     if not overlap_end:
-        raise_if_not(start + series.freq() * forecast_horizon in series,
+        raise_if_not(start + series.freq * forecast_horizon in series,
                      '`start` timestamp is too late in the series to make any predictions with'
                      '`overlap_end` set to `False`.', logger)
+
+
+def _parallel_apply(iterator: Iterator[Tuple], fn: Callable, n_jobs: int, fn_args, fn_kwargs) -> List:
+    """
+    Utility function that parallelise the execution of a function over an Iterator
+
+    Parameters
+    ----------
+    iterator (Iterator[Tuple])
+        Iterator which returns tuples of input value to feed to fn. Constant `args` and `kwargs` should passed through
+        `fn_args` and  `fn_kwargs` respectively.
+    fn (Callable)
+        The function to be parallelized.
+    n_jobs (int)
+        The number of jobs to run in parallel. Defaults to `1` (sequential). Setting the parameter to `-1` means using
+        all the available processors.
+        Note: for a small amount of data, the parallelisation overhead could end up increasing the total
+        required amount of time.
+    fn_args
+        Additional arguments for each `fn()` call
+    fn_kwargs
+        Additional keyword arguments for each `fn()` call
+
+    """
+
+    returned_data = Parallel(n_jobs=n_jobs)(delayed(fn)(*sample, *fn_args, **fn_kwargs)
+                                            for sample in iterator)
+    return returned_data
